/******************* PORTSCN1.CMD - Rexx CGI script ***************************************/
/*** Scanning well known TCP/IP ports of one server for active services. ******************/
/*** !!! CHECK USER PARAMETERS SECTION ****************************************************/
/******************************************************************************************/

/************************* loading DLLs *********************************/
/* loading Rexx Socket API *****************************/
function = RxFuncQuery('SockLoadFuncs')
if function \= 0 then do
   rc = RxFuncAdd('SockLoadFuncs','RXSOCK','SockLoadFuncs')
   rc =  SockLoadFuncs() 
   end
else nop
/* loading Rexx Ftp API *****************************/
function = RxFuncQuery('FtpLoadFuncs')
if function \= 0 then do
   rc = RxFuncAdd('FtpLoadFuncs','rxFtp','FtpLoadFuncs')
   rc =  FtpLoadFuncs('ANY') 
   end
else nop
/****************** end of loading DLLs **********************************/

/******** CGI environment variables **************************************************X******/
env = "OS2ENVIRONMENT"
method = value("REQUEST_METHOD",,env)
client_addr = value("REMOTE_ADDR",,env)
/********************************************************************************************/

/************************* USER CHANGEABLE PARAMETERS *********************************/
/************************* User parameters ********************************************/
/** You can change The Full Path or/and name of SERVICES file in your ETC directory ***/
/** For example service_file_name = 'D:\ETC\SERVICES'                               ***/
service_file_name = value('ETC',,'os2environment')||'\services'
/********************** End of user parameters ****************************************/
/**************************************************************************************/

if method = 'GET' then do
   query_string = value("QUERY_STRING",,env)
   end
else do
   len = value("CONTENT_LENGTH",,env)
   query_string = charin(,,len) /* read only the number specified in CONTENT_LENGTH */
   end

query_string = DecodeKeyVal(query_string)

parse value query_string with 'SELECTION=' selection '&IP_ADDRESS=' ip_address

select
   when selection = 'LISTING_PORTS' then call listing_ports
   otherwise call perform_scan
end

call perform_scan

/************************ END OF MAIN PART ********************************************/

/************************ SUBROUTINES *************************************************/
/************************ perform_scan ************************************************/
perform_scan:

ip_address = strip(ip_address)
action = 'NO'

/******************* checking if IP address is correct ********************************/
if ip_address = '' then do
   error_line = 'Enter valid IP address or Host name.'
   call return_html
end

if pos(':',ip_address) \= 0 then do
   parse value ip_address with ip_address_part ':' given_port
   ip_address_part = strip(ip_address_part) ; given_port = strip(given_port)
   if datatype(given_port) \= 'NUM' then do
      error_line = "Incorrect Host port:" given_port ", it must be number."
      call return_html
   end
   else ip_address = ip_address_part
end
else given_port = ' '

if verify(ip_address,'1234567890.') > 0 then do
   type_address = 'HOSTNAME'
   /*** resolve IP address by host name ****/
   rc = SockGetHostByName(ip_address,'host._')
   if rc \= 1 then do
      error_line = "Unknown host:" ip_address ", can't resolve it's IP address."
      call return_html
   end
   else nop
end
else do
   parse value ip_address with p1 '.' p2 '.' p3 '.' p4
   if p1 = ' ' | p2 = ' ' | p3 = ' ' | p4 = ' ' then do
      error_line = 'Incorect dotted IP address:' ip_address ', try again.'
      call return_html
   end
   type_address = 'IPADDRESS'
   host._addr = ip_address
end
/************ end of checking IP address **********************************************/

/******* reading SERVICES file in ETC directory ********************************************/
/******** perform reading Services file only for selection SCAN_ONLY and PING_SCAN *********/
if selection = 'PING_ONLY'then nop
else do
   port_num. = '' ; protokol. = '' ; service. = ''
   num_serv = 0
   given_protokol = ' ' ; given_service = 'Unknown'

   /***** Exist Service file ? ***/
   exist = stream(service_file_name,'C','QUERY EXISTS')
   if exist = ' ' then do
      error_line = "Your 'SERVICES' file:" service_file_name "doesn't exist, check setting for it",
                   "in 'User parameters' section PORTSCN.CMD"
      call return_html
   end

   do while lines(service_file_name) \= 0
      line = linein(service_file_name)
      if substr(line,1,1) = '#' | line = '' then nop
      else do
         parse var line part1 part2
         parse value part2 with port_num '/' protokol service
         port_num = strip(port_num)
         if datatype(port_num) \= 'NUM' then iterate /*** nonnumerical port number ***/
         protokol = strip(protokol)
         protokol = translate(protokol)
         /*** protokol should be 'udp' or 'tcp' ***/
         if protokol = ' ' | (protokol \= 'TCP' & protokol \= 'UDP') then iterate
         if service = ' ' | service = '#' then service = strip(part1)
         if service = ' ' then service = 'Unknown service'
         service = strip(service)
         service = space(translate(service,' ','#'))
         num_serv = num_serv + 1
         port_num.num_serv = port_num ; protokol.num_serv = protokol ; service.num_serv = service
         if given_port \= ' ' & given_port = port_num & protokol = 'TCP' then do
            given_protokol = protokol ; given_service = service
         end
      end
   end
   port_num.0 = num_serv ; protokol.0 = num_serv ; service.0 = num_serv
   x = lineout(service_file_name) /*** close file ***/
end
if given_port \= ' ' then do
   num_serv = 1 ; port_num.1 = given_port ;  protokol.1 = given_protokol ; service.1 = given_service
end
/******* end of reading SERVICES file ****************************************************/

/******************************* First PING to server ************************************/
/******** Perform Ping only for selection PING_ONLY and PING_SCAN ************************/
if selection = 'PING_ONLY' | selection = 'PING_SCAN' then do
   bytes = 10
   do png = 1 to 3
      sec = FtpPing(host._addr,bytes)
      if datatype(sec) = 'NUM' then png = 4
   end
   if datatype(sec) = 'NUM' then do
      if selection = 'PING_ONLY' then do
         if sec = 0 then sec = 'less then 10'
         error_line = 'Host:' ip_address ',response to Ping OK, response time:' sec 'milisec.'
         call return_html
      end
   end
   else do
      select
         when sec = 'PINGREPLY' then,
            error_line = 'Server:' ip_address 'does not reply to PING.'
         when sec = 'PINGSOCKET' then,
            error_line = 'Ping error, unable to obtain socket.'
         when sec = 'PINGSEND' then,
            error_line = 'Ping error, send failed.'
         when sec = 'PINGRECV' then,
            error_line = 'Ping error, receive failed.' 
         when sec = 'PINGHOST' then,
            error_line = 'Ping error, unknown host on IP address:' ip_address
         otherwise,
            error_line = 'PING - undefined error.'
      end
      call return_html
   end
end
else nop
/*********************** end of Ping ************************************************/

/************ create socket and try connect to port *********************************/

Host._family = 'AF_INET' 
ht_line1. = '' ; ht_line2. = '' ; ht_line3. = '' ; ht_line4. = '' ; ht_line5. = '' ; ht_num = 0

/****** Loop based on reading service file *********/
do n = 1 to num_serv
   port_name = protokol.n
   if port_name = 'TCP' then do
      Host._port = port_num.n
      call create_socket
      if continue = 'NO' then call return_html
      else nop
      call connect_socket
      if continue = 'NO' then nop
      else do
         ht_num = ht_num + 1
         ht_line1.ht_num = 'Found service on port:' port_num.n 
         ht_line2.ht_num = 'protocol:' protokol.n 
         ht_line3.ht_num = 'Type of service:' service.n
         ht_line4.ht_num = aktualni_stav
         ht_line5.ht_num = elapsed_time
      end
      call close_socket
      if continue = 'NO' then call return_html
      else nop
   end
   else nop
end

action = 'YES'
if ht_num = 0 then nop
else do
   ht_line. = ''
   ht_line.1 = '<table border=7 BGCOLOR="#FFFFFF">'
   ht_line.2 = '<TR><TH><TH Align="center"> Server port<TH Align="center"> Protocol <TH Align="center"> Name-type of service <TH Align="center"> Connected <TH Align="center"> Response time</TR>'
   ht = 2
   do n = 1 to ht_num
      ht = ht + 1
      ht_line.ht = '<TR><TD Align="left"><TD Align="left">' ht_line1.n '<TD Align="left">' ht_line2.n '<TD Align="left">' ht_line3.n '<TD Align="left">' ht_line4.n '<TD Align="left">' ht_line5.n '</TR>'
      end
   ht = ht + 1
   ht_line.ht = '</TABLE>'
   end

if selection = 'SCAN_ONLY' then do
   error_line = 'Port scanner succesfully finished scanning of IP address :' ip_address
   end
else do
   if sec = 0 then sec = 'less then 10'
   error_line = 'Port scanner succesfully finished scanning of IP address :' ip_address '<br>',
                'Ping response time:' sec 'milisec.'
   end
call return_html

/**** connect_socket - connect to port **************************************/
connect_socket:
continue = 'NO'

x = time('r')
rc = SockConnect(Socket,'Host._')
elapsed_time = time('e')
elapsed_time = format(elapsed_time,,2) 'sec.'

if rc = -1 then do
   type_errno = errno
   select
      when type_errno = 'ECONNREFUSED' then nop
      when type_errno = 'ETIMEDOUT' then do
         error_line = 'The connection establishment to' ip_address 'timed out before a',
                      'connection was made.'
         call close_socket
         call return_html
      end
      when type_errno = 'ENETUNREACH' then do
         error_line = 'The network of' ip_address 'cannot be reached from this host.'
         call close_socket
         call return_html
      end
      when type_errno = 'EADDRNOTAVAIL' then do
         error_line = 'The calling host cannot reach the specified destination:' ip_address
         call close_socket
         call return_html
      end
      when type_errno = 'EAFNOSUPPORT' then do
         error_line = 'ERROR - The address family:' Host._family 'is not supported.'
         call close_socket
         call return_html
      end
      otherwise do
         error_line = 'Connect error, errno:' type_errno ', h_errno:' h_errno ', Host address:' ip_address
         call close_socket
         call return_html
      end
   end
end
else do
   aktualni_stav = 'OK' ; continue = 'YES'
end
return

/**** create_socket - create new socket ******************************************/
create_socket:
domain = 'AF_INET' ; type = 'SOCK_STREAM' ; protocol = 0
continue = 'NO'
Socket = SockSocket(domain,type,protocol)
if Socket = '-1' then do
   error_line = 'Error funkction SockSocket(), errno:' errno 'h_errno:' h_errno
end
else continue = 'YES'
return

/**** close_socket subrutina pro close a shutdown socket-u ************************/
close_socket:
how = 2
continue = 'NO'
rc = SockShutDown(Socket,how)
rc = SockClose(Socket)
if rc = -1 then do
   error_line = 'Error funkction SockClose() ,errno' errno 'h_errno:' h_errno
end
else continue = 'YES'
return

/************************************* return_html **********************************************/
return_html:
Say 'Content-type: text/html'
Say
say '<html><head><title> Port Scanner </title>'
say '<SCRIPT LANGUAGE="JavaScript">'
say '<!-- Begin'
say '// ***************************************************'
say '// noEntry funkction'
say '// ***************************************************'
say 'function noEntry()'
say '{'
say '// ***** deklarace promennych'
say 'mt1=document.FORM001.IP_ADDRESS.value;'
say '// **** check if IP address is present ****'
say 'if ((mt1.length<1)||(mt1.substring(0,3)=="???")) {'
say 'alert("Enter valid IP address of the server !");'
say 'document.FORM001.IP_ADDRESS.value="???";'
say 'document.FORM001.IP_ADDRESS.focus();return false'
say '}'
say 'return true;'
say '}'
say '// ******************** end of funkction noEntry **************************'
say '// End -->'
say '</SCRIPT>'
say '</head>'
say '<BODY BGCOLOR="#00BFBF" TEXT="#000000" LINK="#0824F8" ALINK="#D014F8" VLINK="#6084E0">'
say '<CENTER>'
say '<H1>Port Scanner.</H1>'
say '<hr size=5>'
say '<B><font color="#FF0000">' error_line '</font></B>'
say '<FORM NAME="FORM001" onSubmit="return noEntry()" ACTION="/cgi-bin/PORTSCN1" METHOD="POST">'
say '<TABLE BORDER=7>'
say '<TR align="LEFT"><TD>'
checked1 = '' ; checked2 = '' ; checked3 = ''
select
   when selection = 'PING_ONLY' then checked1 = 'CHECKED'
   when selection = 'SCAN_ONLY' then checked2 = 'CHECKED'
   otherwise checked3 = 'CHECKED'
end
say '<INPUT TYPE="radio" NAME="SELECTION" VALUE="PING_ONLY" 'checked1||'> Ping Host only'
say '<BR><INPUT TYPE="radio" NAME="SELECTION" VALUE="SCAN_ONLY" 'checked2||'> Scan Host ports only'
say '<BR><INPUT TYPE="radio" NAME="SELECTION" VALUE="PING_SCAN" 'checked3||'> Ping Host and scan Host ports'
say '</TD></TR>'
say '<TR align="LEFT"><TD>'
say '<I> Optionally you can specify Host port for scanning, for example port 80 - </i><b>xxx.yyy.zzz:80 </b><br><br>'
say '<b>IP address/Host name of the server:</b>'
say '<INPUT NAME="IP_ADDRESS" VALUE='|| '"'ip_address'"' ||'SIZE=40><br>'
say '</TD></TR>'
say '</table>'
say '<br><TABLE BORDER=8 CELLSPACING=0 CELLPADDING=0>'
say '<TR><TD><INPUT TYPE="SUBMIT" VALUE="Start scanning"></TD>'
say '</FORM NAME="FORM001">'

say '<FORM NAME="FORM002">'
say '<TD>'
say "<INPUT TYPE="'"BUTTON"'" VALUE="'"Listing well known ports"'" onclick=""window.open('/cgi-bin/PORTSCN1?SELECTION=LISTING_PORTS','listing_ports','width=400,height=500,scrollbars')"">"
say '</TD>'
say '</FORM NAME="FORM002">'

say '</TR></TABLE>'
say '<hr size=5>'
say '<br>'

if action = 'NO' then nop
else do
   if ht_num = 0 then do
      if given_port = ' ' then,
           say '<H3> On IP address:' ip_address 'is not active any Server/service on Well Known ports</H3>'
      else say '<H3> IP address:' ip_address ', there is not active service on port:' given_port '</H3>'
   end
   else do
      say '<H3> Active services on IP address' ip_address ' :</h3>'
      do n = 1 to ht
         say ht_line.n
      end
   end
   say '<hr size=5>'
end

say '<b> Request from IP address:' client_addr '</b><br>'
say '</body></html>'
exit

/************************************ Listing_ports ********************************************/
listing_ports:

/******* reading SERVICES file in ETC directory for listing ports ******************************/

ht_num = 0

/***** Exist Service file ? ***/
exist = stream(service_file_name,'C','QUERY EXISTS')
if exist = ' ' then do
   error_line = "Your 'SERVICES' file:" service_file_name "doesn't exist, check setting for it",
                "in 'User parameters' section PORTSCN.CMD"
   call return_listing_html
end

do while lines(service_file_name) \= 0
   line = linein(service_file_name)
   if substr(line,1,1) = '#' | line = '' then nop
   else do
      parse var line part1 part2
      parse value part2 with port_num '/' protokol service
      port_num = strip(port_num)
      if datatype(port_num) \= 'NUM' then iterate /*** nonnumerical port number ***/
      protokol = strip(protokol)
      protokol = translate(protokol)
      /*** protokol should be 'udp' or 'tcp' ***/
      if protokol = ' ' | (protokol \= 'TCP' & protokol \= 'UDP') then iterate
      if service = ' ' | service = '#' then service = strip(part1)
      if service = ' ' then service = 'Unknown service'
      service = strip(service)
      service = space(translate(service,' ','#'))
      ht_num = ht_num + 1
      ht_line.ht_num = '<LI> Port:' port_num '-' service 'Protocol:' protokol
/*    port_num.num_serv = port_num ; protokol.num_serv = protokol ; service.num_serv = service*/
   end
end
x = lineout(service_file_name) /*** close file ***/
if ht_num = 0 then do
   error_line = "Problem with Your 'SERVICES' file:" service_file_name ", it looks to be empty."
   call return_listing_html
end
else error_line = ' '
/******* end of reading SERVICES file ****************************************************/

/******************************** return_listing_html ****************************************/
return_listing_html:
Say 'Content-type: text/html'
Say
say '<html><head><title> Listing wel known ports </title></head>'
say '<BODY BGCOLOR="#00BFBF" TEXT="#000000" LINK="#0824F8" ALINK="#D014F8" VLINK="#6084E0">'
say '<CENTER>'
say '<H3>Listing well known ports.</H3>'
say '<hr size=5>'
say '<B><font color="#FF0000">' error_line '</font></B>'

if ht_num = 0 then nop
else do
   say '</center><OL>'
   do n = 1 to ht_num
      say ht_line.n
   end
   say '</OL><center>'
end

say '</BODY></HTML>'
exit

/************************* dekoding query_string **********************************/
/* Do not modify below this line --  Important parsing code... */
ParseQueryString: procedure expose Parms. NFields
Parse arg P
i = 1
do while ((P \= '') & (i < 10))
   Parse var P Parms.Text.i '&' rest
   Parse var Parms.Text.i Parms.Tag.i '=' Parms.KeyVal.i
   Parms.Tag.i = translate( Parms.Tag.i)
   Parms.XVal.i=DecodeKeyVal( Parms.KeyVal.i)
   P = rest
   i = i + 1
   end
NFields = i - 1
return NFields

DecodeKeyVal: procedure
parse arg Code
Text=''
Code=translate(Code, ' ', '+')
rest='%'
do while (rest\='')
   Parse var Code T '%' rest
   Text=Text || T
   if (rest\='') then
   do
      ch = left( rest,2)
      c=X2C(ch)
      Text=Text || c
      Code=substr( rest, 3)
      end
   end
return Text

